# Copyright (C) 2020-2025 Soleta Networks <info@soleta.eu>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

from PyQt6.QtWidgets import (
    QWidget, QPushButton, QVBoxLayout, QHBoxLayout, QLabel, QToolButton, QFrame
)
from PyQt6.QtCore import Qt, QSize, pyqtSignal
from PyQt6.QtGui import QPixmap, QIcon
from src.utils.probe import OSFamily
from src.kiosk.config import *
from src.kiosk.theme import *
from src.utils.net import *
import os


class BootView(QWidget):

    command_requested = pyqtSignal(dict)

    def request_command(self, payload):
        self.command_requested.emit(payload)

    def __init__(self):
        super().__init__()

        self.checked_os = None

        self.layout = QVBoxLayout()
        self.layout.setSpacing(10)
        self.layout.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # Client data frame
        self.data_widget = QFrame(self)
        self.data_widget.setFrameStyle(QFrame.Shape.StyledPanel | QFrame.Shadow.Plain)
        data_layout = QVBoxLayout(self.data_widget)
        data_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)

        ethernet_interface = get_ethernet_interface()
        client_ip = getifaddr(ethernet_interface)
        client_mac = get_mac_from_interface(ethernet_interface)
        client_hostname = os.uname().nodename
        client_link = ethtool(ethernet_interface)

        label_data_list = [
            f'IP: {client_ip}',
            f'MAC: {client_mac}',
            f'HOSTNAME: {client_hostname}',
            f'LINK: {client_link} Mbps'
        ]

        for label_text in label_data_list:
            data_label = QLabel(label_text)
            data_label.setStyleSheet('font-weight: 600;')
            data_layout.addWidget(data_label)

        self.data_widget.setParent(self)
        self.data_widget.adjustSize()
        self.data_widget.move(0, 0)

        # Header
        image_label = QLabel()
        image_label.setStyleSheet("background: transparent;")
        pixmap = get_image('header.png')
        image_label.setPixmap(pixmap)
        image_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.layout.addWidget(image_label)
        self.layout.addStretch()

        boot_os_list = get_boot_os_list()

        self.restore_button = QPushButton(self.tr(' Restore'))
        self.boot_button = QPushButton(self.tr(' Boot'))

        self.os_layout = QVBoxLayout()
        self.os_layout.setSpacing(2)
        self.os_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.layout.addLayout(self.os_layout)
        os_row_layout = None

        win_icon = get_icon('windows.png')
        linux_icon = get_icon('linux.png')

        for part_idx, part_data in enumerate(boot_os_list):
            if not os_row_layout or os_row_layout.count() == get_os_columns():
                os_row_layout = QHBoxLayout()
                os_row_layout.setSpacing(2)
                os_row_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
                self.os_layout.addLayout(os_row_layout)

            button = QToolButton()
            button.setText(' {os_name} ({os_disk},{os_part})'.format(
                os_name=part_data['name'],
                os_disk=part_data['disk'],
                os_part=part_data['partition']
            ))
            button.setStyleSheet(f'height: {BUTTON_HEIGHT}px; width: {BUTTON_WIDTH}px;')
            button.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextUnderIcon)
            button.setCheckable(True)
            if part_data['os'] == OSFamily.WINDOWS.name:
                button.setIcon(win_icon)
            elif part_data['os'] == OSFamily.LINUX.name:
                button.setIcon(linux_icon)
            button.setIconSize(QSize(BUTTON_ICON_SIZE, BUTTON_ICON_SIZE))
            button.toggled.connect(lambda checked, b=button: self.on_part_button_checked(checked, b))
            button.setProperty('part_idx', part_idx)
            os_row_layout.addWidget(button)

            if not self.checked_os:
                button.setChecked(True)

        power_layout = QHBoxLayout()
        power_layout.setSpacing(10)
        power_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.layout.addLayout(power_layout)

        # Reboot
        reboot_button = QPushButton()
        reboot_button.setStyleSheet(f'height: {BUTTON_HEIGHT}px; width: {BUTTON_HEIGHT}px;')
        icon = get_icon('rotate-right.png')
        reboot_button.setIcon(icon)
        reboot_button.setIconSize(QSize(POWER_ICON_SIZE, POWER_ICON_SIZE))
        power_layout.addWidget(reboot_button)
        reboot_button.clicked.connect(self.on_reboot_pressed)

        boot_qss = f'height: {BUTTON_HEIGHT - 6}px; width: {BUTTON_HEIGHT * 2}px; font-size: 22px;'

        # Restore
        icon = get_icon('compact-disc.png')
        self.restore_button.setIcon(icon)
        self.restore_button.setStyleSheet(boot_qss)
        power_layout.addWidget(self.restore_button)
        self.restore_button.clicked.connect(self.on_restore_pressed)
        self.restore_button.setVisible(len(boot_os_list) > 0)

        # Boot
        icon = get_icon('display.png')
        self.boot_button.setIcon(icon)
        self.boot_button.setStyleSheet(boot_qss)
        power_layout.addWidget(self.boot_button)
        self.boot_button.clicked.connect(self.on_boot_os_pressed)
        self.boot_button.setVisible(len(boot_os_list) > 0)

        # Poweroff
        poweroff_button = QPushButton()
        icon = get_icon('power-off.png')
        poweroff_button.setIcon(icon)
        poweroff_button.setIconSize(QSize(POWER_ICON_SIZE, POWER_ICON_SIZE))
        poweroff_button.setStyleSheet(f'height: {BUTTON_HEIGHT}px; width: {BUTTON_HEIGHT}px;')
        power_layout.addWidget(poweroff_button)
        poweroff_button.clicked.connect(self.on_poweroff_pressed)

        # Footer
        self.layout.addStretch()
        image_label = QLabel()
        image_label.setStyleSheet("background: transparent;")
        pixmap = get_image('footer.png')
        image_label.setPixmap(pixmap)
        image_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.layout.addWidget(image_label)

        self.setLayout(self.layout)


    def on_part_button_checked(self, checked, checked_button):
        partition_has_image = False
        if checked:
            if self.checked_os and self.checked_os != checked_button:
                self.checked_os.setChecked(False)
            self.checked_os = checked_button
            part_idx = checked_button.property('part_idx')
            if part_idx is not None:
                part_data = get_boot_os_list()[part_idx]
                partition_has_image = bool(part_data.get('image', ''))
        else:
            self.checked_os = None

        self.boot_button.setEnabled(checked)
        self.restore_button.setEnabled(checked and partition_has_image)

    def on_reboot_pressed(self):
        payload = {'command': 'reboot'}
        self.request_command(payload)

    def on_poweroff_pressed(self):
        payload = {'command': 'poweroff'}
        self.request_command(payload)

    def on_boot_os_pressed(self):
        part_idx = self.checked_os.property('part_idx')
        part_data = get_boot_os_list()[part_idx]
        payload = {
            'command': 'boot',
            'disk': part_data['disk'],
            'partition': part_data['partition']
        }
        self.request_command(payload)

    def on_restore_pressed(self):
        part_idx = self.checked_os.property('part_idx')
        part_data = get_boot_os_list()[part_idx]
        payload = {
            'command': 'restore',
            'image': part_data['image'],
            'disk': part_data['disk'],
            'partition': part_data['partition']
        }
        self.request_command(payload)
