# Copyright (C) 2020-2025 Soleta Networks <info@soleta.eu>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

from PyQt6.QtCore import Qt, pyqtSignal, QCoreApplication
from PyQt6.QtWidgets import (
    QWidget, QPushButton, QVBoxLayout, QFrame
)
from PyQt6.QtGui import QPixmap, QIcon
from src.kiosk.monitor import MonitorView
from src.kiosk.boot import BootView
from src.kiosk.config import *
from src.kiosk.theme import *
from enum import Enum


class ViewType(Enum):
    SYSTEM_MONITOR = 1
    BOOT_OS = 2

    def get_view_title(self):
        if self == ViewType.SYSTEM_MONITOR:
            return QCoreApplication.translate('ViewType', 'System monitor')
        elif self == ViewType.BOOT_OS:
            return QCoreApplication.translate('ViewType', 'Boot OS')
        else:
            return ''

    def get_view_icon(self):
        if self == ViewType.SYSTEM_MONITOR:
            icon_name = 'chart-simple.png'
        elif self == ViewType.BOOT_OS:
            icon_name = 'display.png'
        else:
            return QIcon()

        return get_icon(icon_name)

    def get_widget_instance(self):
        if self == ViewType.SYSTEM_MONITOR:
            return MonitorView()
        elif self == ViewType.BOOT_OS:
            return BootView()
        else:
            return None


class SidePanel(QFrame):

    widget_change_requested = pyqtSignal(QWidget)

    def __init__(self):
        super().__init__()

        self.panel_hidden = False
        self.last_widget = ViewType.BOOT_OS
        self.button_map = {}

        self.setFrameStyle(QFrame.Shape.StyledPanel | QFrame.Shadow.Plain)

        layout = QVBoxLayout()
        layout.setAlignment(Qt.AlignmentFlag.AlignTop)
        self.setLayout(layout)

        for view_type in ViewType:
            button = QPushButton()
            button.setText(' ' + view_type.get_view_title())
            button.setStyleSheet('height: 40px;')
            button.setIcon(view_type.get_view_icon())
            layout.addWidget(button)
            button.clicked.connect(lambda _, view=view_type: self.request_widget_change(view))

            self.button_map[view_type] = button

    def reload_theme(self):
        for view_type, button in self.button_map.items():
            button.setIcon(view_type.get_view_icon())

    def request_widget_change(self, view_type):
        self.last_widget = view_type
        self.widget_change_requested.emit(view_type.get_widget_instance())
